/***************************************************************************
                          client.cpp  -  description
                             -------------------
    begin                : Sat Sep 14 2002
    copyright            : (C) 2002 by Ivica Bukvic
    email                : ico@fuse.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "client.h"
#include "rtmix.h"
#include "consolewindow.h"
#include "externaltimer.h"

#include <qsocket.h>
#include <qserversocket.h>
#include <qtextstream.h>
#include <qtimer.h>

Client::Client( RTMix *r, QString s, QString host, Q_UINT16 port )
{
	//connect( send, SIGNAL(clicked()), SLOT(sendToServer()) );
	//connect( close, SIGNAL(clicked()), SLOT(closeConnection()) );

	//Set the name for this connection and reset all variables
	name = s;
	ptr = r;
	Host = host;
	Port = port;
	line = 0;
	Status = 0;

	// create the socket and connect various of its signals
	socket = new QSocket( this );
	connect( socket, SIGNAL(connected()), SLOT(socketConnected()) );
	connect( socket, SIGNAL(connectionClosed()), SLOT(socketConnectionClosed()) );
	//connect( socket, SIGNAL(readyRead()), SLOT(socketReadyRead()) );
	connect( socket, SIGNAL(error(int)), SLOT(socketError(int)) );
}

Client::~Client()
{
	closeConnection();
}

void Client::createConnection()
{
	//qDebug( "%i", Status );
	if ( Status < 2 )
	{
		//connect to the server
		socket->connectToHost( Host, Port );
		//qDebug( Host + ":%i", Port );
		//Status = 2;
	}
}

void Client::closeConnection()
{
	if ( Status > 0 )
	{
        socket->close();
        if ( socket->state() == QSocket::Closing )
        {
        	// We have a delayed close.
        	connect( socket, SIGNAL(delayedCloseFinished()), SLOT(socketClosed()) );
        }
        else
        {
        	// The socket is closed.
        	socketClosed();
        }
	}
}

void Client::sendStringToServer( QString s )
{
    // write to the server
    //ptr->T->ToggleANetO( TRUE );
    QTextStream os(socket);
    os << s;
}

void Client::sendToServer( struct eventCue *e )
{
   // write to the server
   //qDebug("Parsing");
    //ptr->T->ToggleANetO( TRUE );
   QString s = ParseNetEvent( e );
   //qDebug("Setting stream");
   QTextStream os(socket);
   //qDebug("Sending");
   os << s;
   //qDebug("Done");
}

QString Client::ParseNetEvent( struct eventCue *e )
{
	QString result, tmp;

	result = "E:" + e->event + "<|>" + e->killEvent + "<|>" + e->text + "<|>" + e->say + "<|>" + tmp.setNum( e->ID, 10 ) + "<|>";
	result = result + tmp.setNum( e->time, 10 ) + "<|>";
	result = result + tmp.setNum( e->countdown, 10 ) + "<|>";
	result = result + tmp.setNum( e->absat, 10 ) + "<|>";
	result = result + tmp.setNum( e->happen, 10 ) + "<|>";
	result = result + tmp.setNum( e->chance, 10 ) + "<|>" + e->color + "<|>";
	result = result + tmp.setNum( e->light, 10 ) + "<|>";
	result = result + tmp.setNum( e->blinkSpeed, 10 ) + "<|>";
	result = result + tmp.setNum( e->prepLength, 10 ) + "<|>";
	result = result + tmp.setNum( e->lightLength, 10 ) + "<|>";
	result = result + "-2<|>" + tmp.setNum( e->bpm, 10 ) + "<|>";
	result = result + tmp.setNum( e->type, 10 ) + "<|>" + e->var + "<|>";
	result = result + tmp.setNum( e->method, 10 ) + "<|>";
	result = result + tmp.setNum( e->amt ) + "<|>" + e->accel + "\n";
	//qDebug ( result );
	return result;
}

void Client::socketConnected()
{
    //SEND AUTH TO SERVER
    EmitText( tr( "Connection no." + name + " reports: connected...\n") );
    ptr->UpdateNetStatus( name.toInt( NULL, 10 ), "ON" );
    Status = 2;
    ptr->T->ToggleMNet( TRUE );
    ptr->T->ToggleANetO( TRUE );
    sendStringToServer( ptr->GetNetAuth() + "\n" );
    //assume half a second should be enough for auth to travel via network
    QTimer::singleShot( 500, this, SLOT( CheckAuth() ) );   
}

void Client::CheckAuth()
{
    if ( Status == 1 )
    {
        EmitText( tr( "Connection no." + name + " reports: bad network authorization?...\n") );
        ptr->UpdateNetStatus( name.toInt( NULL, 10 ), "BAD_AUTH" );
    }
}

void Client::socketConnectionClosed()
{
    EmitText( tr( "Connection no." + name + " reports: connection closed by the other end (you will need to explicitly reactivate it, or explicitly disable it before trying to reconnect via scriptfile)...\n") );
    ptr->UpdateNetStatus( name.toInt( NULL, 10 ), "CLOSED" );
	Status = 1;
}

void Client::socketClosed()
{
    EmitText( tr( "Connection no." + name + " reports: connection closed...\n") );
    ptr->UpdateNetStatus( name.toInt( NULL, 10 ), "OFF" );
	Status = 0;
}

void Client::socketError( int e )
{
	QString tmp;
    EmitText( tr( "Connection no." + name + " reports: error number %1 occurred...\n").arg(e) );
    ptr->UpdateNetStatus( name.toInt( NULL, 10 ), "ERROR " + tmp.setNum( e ) );
	Status = 1;
}
