/***************************************************************************
                          metro.cpp  -  description
                             -------------------
    begin                : Mon Aug 12 2002
    copyright            : (C) 2001 by Ivica Bukvic
    email                : ico@fuse.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "metro.h"

#include <qframe.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qimage.h>
#include <qpixmap.h>

/* 
 *  Constructs a Metronome which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 */
Metronome::Metronome( QWidget* parent,  const char* name, WFlags fl )
    : QWidget( parent, name, fl )
{

	//LED.load("/usr/local/rtmix/images/redon.xpm");
	//LEDoff.load("/usr/local/rtmix/images/redoff.xpm");
	LED = new QPixmap();
	LEDoff = new QPixmap();
	Spacer = new QPixmap();

	Spacer->load("/usr/local/rtmix/images/blankled.xpm");
    Spacer->fill( paletteBackgroundColor() );

    if ( !name )
    setName( "Metronome" );
    //setSizeIncrement( 10, 10 );
    setCaption( tr( "Metro" ) );
    MetronomeLayout = new QVBoxLayout( this );
    //MetronomeLayout->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)1, (QSizePolicy::SizeType)1, FALSE ) );
    MetronomeLayout->setSpacing( 6 );
    MetronomeLayout->setMargin( 6 );

    WindowLabel = new QLabel( this, "WindowLabel" );
    WindowLabel->setMinimumSize( QSize( 120, 30 ) );
    WindowLabel->setMaximumSize( QSize( 32767, 30 ) );
    QFont WindowLabel_font(  WindowLabel->font() );
    WindowLabel_font.setPointSize( 10 );
    WindowLabel->setFont( WindowLabel_font );
    WindowLabel->setText( tr( "Metro" ) );
    WindowLabel->setAlignment( int( QLabel::AlignCenter ) );

	MetronomeLayout->addWidget( WindowLabel, 0 );

    LEDLayout = new QGridLayout();
    LEDLayout->setSpacing( 0 );
    LEDLayout->setMargin( 0 );

    LEDLabel = new QLED( this, "LED", WResizeNoErase | WRepaintNoErase );
    LEDLabel->setMinimumSize( QSize( 30, 30 ) );
    LEDLabel->setMaximumSize( QSize( 120, 120 ) );
	//LEDLabel->setUpdatesEnabled( FALSE );
    //LEDLabel->setPixmap( *LEDoff );
    LEDLabel->setScaledContents( TRUE );

	StartLED = LEDLabel;
	MoveLED = LEDLabel;

    LEDLayout->addWidget( LEDLabel, 0, 0 );

    LEDLabel = new QLED( this, "LED", WResizeNoErase | WRepaintNoErase );
    LEDLabel->setMinimumSize( QSize( 30, 30 ) );
    LEDLabel->setMaximumSize( QSize( 120, 120 ) );
	//LEDLabel->setUpdatesEnabled( FALSE );
    //LEDLabel->setPixmap( *LEDoff );
    LEDLabel->setScaledContents( TRUE );

    LEDLayout->addWidget( LEDLabel, 0, 1 );
	MoveLED->Ptr = LEDLabel;
	MoveLED = LEDLabel;

    LEDLabel = new QLED( this, "LED", WResizeNoErase | WRepaintNoErase );
    LEDLabel->setMinimumSize( QSize( 30, 30 ) );
    LEDLabel->setMaximumSize( QSize( 120, 120 ) );
	//LEDLabel->setUpdatesEnabled( FALSE );
    //LEDLabel->setPixmap( *LEDoff );
    LEDLabel->setScaledContents( TRUE );

    LEDLayout->addWidget( LEDLabel, 0, 2 );
	MoveLED->Ptr = LEDLabel;
	MoveLED = LEDLabel;

    LEDLabel = new QLED( this, "LED", WResizeNoErase | WRepaintNoErase );
    LEDLabel->setMinimumSize( QSize( 30, 30 ) );
    LEDLabel->setMaximumSize( QSize( 120, 120 ) );
	//LEDLabel->setUpdatesEnabled( FALSE );
    //LEDLabel->setPixmap( *LEDoff );
    LEDLabel->setScaledContents( TRUE );

    LEDLayout->addWidget( LEDLabel, 0, 3 );
	MoveLED->Ptr = LEDLabel;
	MoveLED = LEDLabel;

	MoveLED = StartLED;

    Line = new QFrame( this, "Line" );
    Line->setFrameStyle( QFrame::HLine | QFrame::Sunken );

    TextLayout = new QHBoxLayout; 
    TextLayout->setSpacing( 6 );
    TextLayout->setMargin( 0 );

    TextMeter = new QLabel( this, "TextMeter" );
    TextMeter->setMinimumSize( QSize( 0, 16 ) );
    TextMeter->setMaximumSize( QSize( 32767, 16 ) );
    TextMeter->setText( tr( "M:4" ) );
    TextMeter->setAlignment( int( QLabel::AlignCenter ) );
    TextLayout->addWidget( TextMeter );

    TextBPM = new QLabel( this, "TextBPM" );
    TextBPM->setMinimumSize( QSize( 0, 16 ) );
    TextBPM->setMaximumSize( QSize( 32767, 16 ) );
    TextBPM->setText( tr( "BPM:060" ) );
    TextBPM->setAlignment( int( QLabel::AlignCenter ) );
    TextLayout->addWidget( TextBPM );

    MetronomeLayout->addLayout( TextLayout );
    MetronomeLayout->addWidget( Line );

	MetronomeLayout->addLayout( LEDLayout );

	BPS = 100;
	Timer = 0;
	LEDNum = 4;
	LEDRow = 1;
	CurrLED = -1;
	SpacerNum = 0;
	Status = 0;
	MaxWidth = 4;
	Title = "Metro";

	//setMinimumSize( QSize( 28 * MaxWidth, 79 + LEDRow * 30 ) );
	setMaximumSize( QSize( 115 * MaxWidth, 79 + LEDRow * 111 ) );
    //qDebug(" test: %d", 79 + LEDRow * 111 );
	resize( QSize( 28 * MaxWidth, 79 + LEDRow * 30 ) );

  imgX = 30;
  imgY = 30;
  Updating = false;
  oldW = 132;
  oldH = 109;
}

/*
 *  Destroys the object and frees any allocated resources
 */
Metronome::~Metronome()
{
    // no need to delete child widgets, Qt does it all for us
}

bool Metronome::event( QEvent* ev )
{
    bool ret = QWidget::event( ev );
    if ( ev->type() == QEvent::Hide )
    {
      emit Closed( 0 );
    }
    if ( ev->type() == QEvent::Resize )
    {
      if ( StartLED != NULL && !Updating )
      {
        imgX = StartLED->width();
        imgY = imgX;
      }
      if ( !ev->spontaneous() && oldW == 0 && oldH == 0 )
      {
          QLED *Temp;
          int loop;
        
          oldW = width();
          oldH = height();
          //qDebug( "resize: %i %i %i %d", width(), height(), ev->type(), ev->spontaneous() );

          Temp = StartLED;
          loop = LEDNum;

          while ( loop )
          {
              Temp->setMaximumSize( 120, 120 );
              loop--;
              if ( loop > 0 )
              {
                  Temp = Temp->Ptr;
              }
          }

          Temp = StartSpacer;
          loop = SpacerNum;

          while ( loop )
          {
              Temp->setMaximumSize( 120, 120 );
              loop--;
              if ( loop > 0 )
              {
                  Temp = Temp->Ptr;
              }
          }
          resize( imgX * MaxWidth + 12, LEDRow * imgX + 79 );
          Updating = false;
          SetResizable( FALSE );
      }
      //qDebug( "test: %i %i %i %d", width(), height(), ev->type(), ev->spontaneous() );
    }
    return ret;
}

void Metronome::UpdateBPS( int val )
{
    Updating = true;
    //qDebug( "var = %d", val );
	QString outVal;

	if ( val < 100 )
	{
     	if ( val < 10 )
		{
			TextBPM->setText( "BPM:00" + outVal.setNum( val, 10 ) );
		}
		TextBPM->setText("BPM:0" + outVal.setNum( val, 10 ) );
	}
	else
	{
	 	TextBPM->setText("BPM:" + outVal.setNum( val, 10 ) );	
	}
	Timer = 0;
	BPS = int(float( 60.0/val ) * float(100.0));
    Updating = false;
}

void Metronome::UpdateName( QString WinName, QString name )
{
    Updating = true;
	if ( WinName != "" )
	{
		setCaption( WinName );
	}
	WindowLabel->setText( name );
    Updating = false;
}

void Metronome::ResetMetro( int type )
{
    Updating = true;
	if ( !type )
	{
		ChangeStatus( 0 );
	}
	MoveLED->setPixmap( *LEDoff );
	CurrLED = -1;
	MoveLED = StartLED;
    Updating = false;
}

int Metronome::UpdateMeter( QString meter )
{
    Updating = true;
	QString value;
	int Row[50];
	bool ok;
	int position, oldPosition, rowNum, displayRow;

	position = 0;
	oldPosition = 0;
	rowNum = 0;
	displayRow = 0;

	if ( meter.find("+", position ) == -1 ) //ONE ELEMENT ONLY
	{
     	Row[ rowNum ] = meter.toInt( &ok, 10 );
		if ( !ok )
		{
			emit MeterParseError( Title );
          Updating = false;
			return -1;  //ERROR
		}
	}

	while ( position != -1 )
	{
		position = meter.find("+", oldPosition + 1 );
		if ( oldPosition == 0 )
		{
	     	value = meter.mid( oldPosition, position );
		}
		else
		{
			value = meter.mid( oldPosition + 1, position - (oldPosition + 1) );
		}
		Row[ rowNum ] = value.toInt( &ok, 10 );
		if ( !ok )
		{
			emit MeterParseError( Title );
          Updating = false;
			return -1;  //ERROR
		}
		if ( position != -1 )
		{
			oldPosition = position;
		}
		rowNum++;	
	}

	//Got this far, this means parsing went fine, so let's change the meter display
	TextMeter->setText( "M:" + meter );	

	//DELETE OLD STUFF
  DeleteDisplay();

	for ( displayRow = 0; displayRow < rowNum; displayRow++ )  //FIND WIDEST ROW
	{
		//LEDNum = LEDNum + Row[ displayRow ];
		if ( MaxWidth < Row[ displayRow ] )
		{
         	MaxWidth = Row[ displayRow ];
		}    	
	}

    LEDLayout = new QGridLayout();
    LEDLayout->setSpacing( 0 );
    LEDLayout->setMargin( 0 );

	for ( displayRow = 0; displayRow < rowNum; displayRow++ )  //FINALLY ADD WIDGETS
	{
		LEDRow++;
     	CreateDisplayRow( Row[ displayRow ], MaxWidth - Row[ displayRow ], displayRow + 1 );
	}
    setMinimumSize( 1, 1 );
    setMaximumSize( 32767, 32767 );    
    resize( 126, 90 );
    oldW = 0;
    oldH = 0;

    MetronomeLayout->addLayout( LEDLayout );
    //LEDLayout->setGeometry( QRect( 0, 0, imgX * MaxWidth, imgX * LEDRow ) );
    //RESIZE WINDOW

    //qDebug( "%d, %d, current w: %d, h: %d", ((imgX + 4) * MaxWidth), (79 + (LEDRow * (imgX + 4))), width(), height() );
    //Updating = false;
    //qDebug( "b %d %d", oldW, oldH );
	return 0;
}

void Metronome::DeleteDisplay()
{
	QLED *Temp;
	
	MoveLED = StartLED;

	if ( LEDNum )
	{
		while ( MoveLED->Ptr != NULL )
		{
			Temp = MoveLED->Ptr;
			delete MoveLED;
			MoveLED = Temp;
		}
		delete MoveLED;
	}

	if ( SpacerNum )
	{
		while ( StartSpacer->Ptr != NULL  )
		{
			Temp = StartSpacer->Ptr;
			delete StartSpacer;
			StartSpacer = Temp;
		}
		delete StartSpacer;
	}

	delete LEDLayout;

	LEDNum = 0;
	SpacerNum = 0;
	CurrLED = -1;
	ChangeStatus( 0 );
	MaxWidth = 1;
	LEDRow = 0;
	Timer = 0;
}

void Metronome::CreateDisplayRow( int l, int s, int r ) //l for LED, s for Spacer, and r for row
{
 	int i, j;
	QLED *Temp;
	QString Name;
  
    //LEDLayout->setRowStretch( r, 0 );

	for ( i = 0; i < l; i++ )
	{
		LEDLabel = new QLED( this, "LED" + Name.setNum( LEDNum, 10 ) );
    	LEDLabel->setMinimumSize( 30, 30 );
    	LEDLabel->setMaximumSize( 30, 30 );
    	LEDLabel->setScaledContents( TRUE );
		LEDLabel->setPixmap( *LEDoff );
       //LEDLabel->resize( imgX, imgY );
       //LEDLayout->setColStretch( i, 0 );

		LEDLayout->addWidget( LEDLabel, r, i );
		LEDLabel->show();

		if ( !LEDNum )
		{
         	StartLED = LEDLabel;
			MoveLED = LEDLabel;
		}
		else
		{
         	MoveLED->Ptr = LEDLabel;
			MoveLED = LEDLabel;
		}

		LEDNum++;	
	}
	for ( j = 0; j < s; j++ )
	{
		SpacerLabel = new QLED( this, "Spacer" );
    	SpacerLabel->setMinimumSize( 30, 30 );
    	SpacerLabel->setMaximumSize( 30, 30 );
    	SpacerLabel->setPixmap( *Spacer );
		SpacerLabel->setScaledContents( TRUE );
       //SpacerLabel->resize( imgX, imgY );
    
		LEDLayout->addWidget( SpacerLabel, r, i + j );
		SpacerLabel->show();
    
		if ( !SpacerNum )
		{
         	StartSpacer = SpacerLabel;
			Temp = SpacerLabel;
		}
		else
		{
         	Temp->Ptr = SpacerLabel;
			Temp = SpacerLabel;
		}

		SpacerNum++;
	}
}

int Metronome::UpdateMetro()
{
	if ( Status == 1 )
	{
        setMaximumSize( size() );
        setMinimumSize( size() );
        resize( size() );  
		if ( BPS == 0 )
		{
         	ChangeStatus( 0 );
			return 0;
		}
		if ( LEDNum > 0 )
		{
			if ( Timer >=  BPS || Timer == 0 )
			{
				Timer = 0;
				if ( CurrLED == -1 )
				{
   		 		 	CurrLED++;
					MoveLED = StartLED;
					MoveLED->setPixmap( *LED );
				}
				else if ( CurrLED == 0 && LEDNum == 1 )
				{
    			 	MoveLED->setPixmap( *LEDoff );
					CurrLED = -1;
				}
				else
				{
					MoveLED->setPixmap( *LEDoff );
					CurrLED++;
					if ( CurrLED == LEDNum )
					{
                	 	CurrLED = 0;
						MoveLED = StartLED;
					}
					else
					{
						MoveLED = MoveLED->Ptr;
					}						
					MoveLED->setPixmap( *LED );
				}
			}
		Timer++;
		}
	}
	return 0;
}

void Metronome::SetImage( QString NamePath, int val )
{
	if ( val == 0 )
	{		
    	LEDoff->load( NamePath, NULL, DiffuseAlphaDither );
		RepaintMetro();
	}
   if ( val == 1 )
	{
    	LED->load( NamePath, NULL, DiffuseAlphaDither );
	}
}

void Metronome::RepaintMetro()
{
 	MoveLED = StartLED;
	while (MoveLED->Ptr != NULL )
	{
		MoveLED->setPixmap( *LEDoff );
		MoveLED = MoveLED->Ptr;
	}
	MoveLED->setPixmap( *LEDoff );
 	MoveLED = StartLED;
}

void Metronome::keyPressEvent ( QKeyEvent *k )
{
    emit KeyPress( k );
}

void Metronome::keyReleaseEvent ( QKeyEvent *k )
{
    emit KeyRelease( k );
}

void Metronome::ChangeStatus( int i )
{
    Status = i;
    if ( Status > 0 )
    {
        SetResizable( FALSE );
    }
    else
    {
        SetResizable( TRUE );
    }
}

void Metronome::SetResizable( bool b )
{
    if ( !b )
    {
        setMinimumSize( size() );
        setMaximumSize( size() );
        //resize( size() );
    }
    else
    {
        setMinimumSize( oldW, oldH );
        //qDebug( "oldH oldW %i %i", oldW, oldH );
        setMaximumSize( 115 * MaxWidth, 79 + LEDRow * 111 );
    }
    //qDebug( "c %i %i", oldW, oldH );
}
