/***************************************************************************
                          simpleserver.cpp  -  description
                             -------------------
    begin                : Sun Sep 15 2002
    copyright            : (C) 2002 by Ivica Bukvic
    email                : ico@fuse.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qtextstream.h>
#include <qtimer.h>

#include "simpleserver.h"
#include "clientsocket.h"
#include "rtmix.h"
#include "externaltimer.h"

SimpleServer::SimpleServer( RTMix *r, QObject* parent, Q_UINT16 prt ) :
        QServerSocket( prt, 1, r )
{
   ptr = r;
   connect( this, SIGNAL ( newConnect( QString ) ), ptr, SLOT ( UpdateHostStatus( QString ) ) );
   for ( int n = 0; n < 256; n++ )
   {
      sock[n] = NULL;
   }
   num = 0;

   if ( !ok() )
   {
      QString tmp;
      emit newConnect("ERROR: Failed to bind port " + tmp.setNum( prt, 10 ) + " (OFF)" );
      ptr->local = NULL;
      //qDebug( "s %i", socket() );
      Status = -1;
      delete this;
   }
   else
   {
      Port = prt;
      //QSocketDevice *sd = socketDevice();
      //sd->setAddressReusable( true );
      Status = 1;
      //for ( int n = 0; n < 256; n++ )
      //{
      // 	sock[n] = NULL;
      //}
      e = NULL;
     EventPtr = new eventCue;
     CountdownPtr = new eventCue;
     TextPtr = new eventCue;
     CheckpointPtr = new eventCue;
     PausePtr = new eventCue;
     StopPtr = new eventCue;
     ClearPtr = new eventCue;
     WarningPtr = new eventCue;
     MetroPtr = new eventCue;
     JumpPtr = new eventCue;
     RandPtr = new eventCue;
     ChangePtr = new eventCue;
     SeedPtr = new eventCue;
     RewPtr = new eventCue;
     FfwPtr = new eventCue;
     ToggleMetroPtr = new eventCue;
     AssignPtr = new eventCue;
     ToggleRTPtr = new eventCue;
     SetNetPtr = new eventCue;
     ToggleNetPtr = new eventCue;
     ToggleMIDIPtr = new eventCue;
     PlayPtr = new eventCue;
          
      emit newConnect( "READY" );
      //qDebug( "s %i, p %i", socket(), port() );
      //qDebug( "Socket Status: %i", sd->addressReusable() );
   }
}

SimpleServer::~SimpleServer()
{
   emit newConnect( "OFF" );
   for ( int n = 0; n < 256; n++ )
   {
      if ( sock[n] != NULL )
      {
         delete sock[n];
      }
   }
   //qDebug( "s %i", socket() );
   //QSocketDevice *sd = socketDevice();
   //if ( sd != NULL )
   //{
   //	sd->close();
   //}
   //qDebug( "s %i", socket() );
}

void SimpleServer::newConnection( int so )
{
   //qDebug( "s %i, p %i", socket(), port() );
   //qDebug( "%i", Status );
   num++;
   sock[num] = new ClientSocket( so, this, num );
   connect ( sock[num], SIGNAL ( Closing() ), this, SLOT ( CloseConnection() ) );
   connect ( sock[num], SIGNAL ( Receiving( bool ) ), this, SIGNAL ( Receiving( bool ) ) );
   connect ( sock[num], SIGNAL ( Sending( bool ) ), this, SIGNAL ( Sending( bool ) ) );
   connect ( sock[num], SIGNAL ( Incoming( int, QString ) ), this, SLOT ( Incoming( int, QString ) ) );
   //INSERT HANDSHAKE CODE:
   connect ( sock[num], SIGNAL ( BadAuth( int ) ), this, SLOT ( CheckAuth( int ) ) );
   //emit newConnect( "ACTIVE" );
   //qDebug( "s %i, p %i", socket(), port() );
   ptr->T->ToggleMNet( TRUE );
   ptr->Error( "Received new network connection..." );
}

void SimpleServer::CheckAuth( int n )
{
    if ( sock[n] != NULL && sock[n]->checkAuth )
    {
          delete sock[n];
          sock[n] = NULL;
          ptr->Error( "WARNING: network client disconnected: no authorization password received..." );
    }
}

void SimpleServer::CloseConnection()
{
   num--;
   //qDebug( "closing %d", num );
   ptr->Error( "Connection terminated by the remote client (master)..." );
   if ( num <= 0 )
   {
      num = 0;
      ptr->T->ToggleMNet( FALSE );
   }
}


int SimpleServer::Incoming( int num, QString s )
{
   int result;
   bool ok;
   QString line, event;

   line = "";
   //qDebug("Incoming");
   if ( s == ptr->LastSent )
   {
      /*
      ptr->Error( ptr->ParseTime() + " -- CRITICAL ERROR: network client disconnected: found infinite recursion within a global network event (is your client connected to itself?)..." );
      delete sock[num];
      sock[num] = NULL;
      CloseConnection();
      */
      //Ignoring since this is a recursive call
      return 0;
   }

   ptr->T->ToggleANetI( TRUE );
   
   QTextStream os( &s, IO_ReadOnly );
   // read from the string
   while ( !os.eof() )
   {
      line = os.readLine();
      //lineNum++;
      //qDebug( "line: " + line );
   }

   if ( sock[num]->checkAuth )
   {
      //qDebug( "||" + line.simplifyWhiteSpace() + "||" );
      //qDebug( "||" + ptr->GetNetAuth().simplifyWhiteSpace() + "||" );
      if ( line.simplifyWhiteSpace() == ptr->GetNetAuth().simplifyWhiteSpace() )
      {
          //connection authorized
          sock[num]->checkAuth = FALSE;
          return 0;
      }
      else
      {
          delete sock[num];
          sock[num] = NULL;
          ptr->Error( "WARNING: network client disconnected due to invalid network authorization password..." );
          CloseConnection();
          return -2;
      }
   }
   
   //qDebug ( line );

   if ( line.find("C:", 0) == 0 )
   {
      //qDebug("C");
      line.remove( 0, 2 );
      //qDebug( line );
      result = line.section("-", 0, 0 ).toInt( &ok, 10 );
      if ( result == 0 ) { ptr->Stop( s ); }
      if ( result == 1 ) { ptr->Stop( s ); }
      if ( result == 3 ) { ptr->Pause( s ); }
      if ( result == 4 ) { ptr->Play( s ); }
   }
   else if ( line.find("J:", 0) ==  0 )
   {
      //take line value and make it change of time
      //qDebug("J");
      line.remove( 0, 2 );
      result = line.section( "-", 0, 0 ).toInt( &ok, 10 );
      //qDebug( "new time %d", result );
      ptr->TimeCount = result;
      ptr->AdjustTimer( s );
      ptr->E->SetTimeCount( result );
   }
   else if ( line.find("P:", 0) == 0 )
   {
      //remotely requested panic button, kills all processes running locally
      line.remove( 0, 2 );
      system ( "killall " + line.section( "-", 0, 0 ) );
   }
   else if ( line.find("E:", 0) == 0 )
   {
      //parse the cue
      event = line;
      event.remove( 0, 2 );
      const QString sep = "<|>";
      //event.section( sep, 1 ); //get rid of the timestamp

      QTextStream eventQts( &event, IO_ReadOnly );
      line = event.section( sep, 4, 4 );
      result = line.toInt();

      switch ( result )
      {
          case 1:
              e = EventPtr;
              break;
          case 2:
              e = CountdownPtr;
              break;
          case 3:
              e = TextPtr;
              break;              
          case 4:
              e = CheckpointPtr;
              break;
          case 5:
              e = PausePtr;
              break;
          case 6:
              e = StopPtr;
              break;
          case 7:
              e = ClearPtr;
              break;
          case 8:
              e = WarningPtr;
              break;
          case 9:
              e = MetroPtr;
              break;
          case 10:
              e = JumpPtr;
              break;
          case 11:
              e = RandPtr;
              break;
          case 12:
              e = ChangePtr;
              break;
          case 13:
              e = SeedPtr;
              break;
          case 14:
              e = RewPtr;
              break;
          case 15:
              e = FfwPtr;
              break;
          case 16:
              e = ToggleMetroPtr;
              break;
          case 17:
              e = AssignPtr;
              break;
          case 18:
              e = ToggleRTPtr;
              break;
          case 19:
              e = SetNetPtr;
              break;
          case 20:
              e = ToggleNetPtr;
              break;
          case 21:
              e = ToggleMIDIPtr;
              break;
          case 22:
              e = PlayPtr;
              break;
      }
      //Fill default values
      /*
      e->event = "none";
      e->killEvent = "none";
      e->text = "none";
      e->say = "none";
      e->ID = 0;
      e->time = 0;
      e->countdown = 0;
      e->absat = 0;
      e->happen = 10000;
      e->chance = 10000;
      e->color = "0";
      e->light = 3;
      e->blinkSpeed = 100;
      e->prepLength = 500;
      e->lightLength = 200;
      e->netAddress = -3;
      e->bpm = 0;
      e->type = 0;
      e->var = "";
      e->method = 0;
      e->amt = 0.0;
      e->accel = "none";
      e->next = NULL;
      e->prev = NULL;
      */
      
      e->event = event.section( sep, 0, 0 );
      //qDebug("event=" + e->event + "\n");
      e->killEvent = event.section( sep, 1, 1 );
      //qDebug("event=" + e->killEvent + "\n");
      e->text = event.section( sep, 2, 2 );
      //qDebug("event=" + e->text + "\n");
      e->say = event.section( sep, 3, 3 );
      //qDebug("event=" + e->say + "\n");
      line = event.section( sep, 4, 4 );
      e->ID = line.toInt();
      //qDebug("event=" + e->event + "\n");
      line = event.section( sep, 5, 5 );
      e->time = line.toInt();
      //qDebug("event=" + e->event + "\n");
      line = eventQts.readLine();
      line = event.section( sep, 6, 6 );
      e->countdown = line.toInt();

      line = event.section( sep, 7, 7 );
      e->absat = line.toInt();

      line = event.section( sep, 8, 8 );
      e->happen = line.toInt();

      line = event.section( sep, 9, 9 );
      e->chance = line.toInt();

      e->color = event.section( sep, 10, 10 );

      line = event.section( sep, 11, 11 );
      e->light = line.toInt();

      line = event.section( sep, 12, 12 );
      e->blinkSpeed = line.toInt();

      line = event.section( sep, 13, 13 );
      e->prepLength = line.toInt();

      line = event.section( sep, 14, 14 );
      e->lightLength = line.toInt();

      line = event.section( sep, 15, 15 );
      e->netAddress = line.toInt();

      line = event.section( sep, 16, 16 );
      e->bpm = line.toInt();               

      line = event.section( sep, 17, 17 );
      e->type = line.toInt();

      e->var = event.section( sep, 18, 18 );

      line = event.section( sep, 19, 19 );
      e->method = line.toInt();             

      line = event.section( sep, 20, 20 );
      e->amt = line.toFloat();
      
      e->accel = event.section( sep, 21, 21 );
      //qDebug("countdown=%i\n", e->countdown);
      //qDebug("lightlength=%i\n", e->lightLength);
      //qDebug("event=" + e->event + "\n");
      //qDebug("HERE");
      ptr->InterpretCue( e );
      //qDebug("THERE");
      if ( e->killEvent != "none" )
      {
        if ( ptr->netEvents == "" )
        {
          ptr->netEvents = "killall " + e->killEvent;
        }                                             
        else ptr->netEvents = ptr->netEvents + " | killall " + e->killEvent;
      }
      return 0;
   }
   else
   {
        ptr->Error( "Network error: illegible message received..." );
        return -1;
   }
   return 0;
}



