/***************************************************************************
                          warning.cpp  -  description
                             -------------------
    begin                : Mon Aug 12 2002
    copyright            : (C) 2002 by Ivica Bukvic
    email                : ico@fuse.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "warning.h"

#include <qlabel.h>
#include <qlcdnumber.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qimage.h>
#include <qpixmap.h>

/* 
 *  Constructs a Warning_Window which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 */
Warning_Window::Warning_Window( QWidget* parent,  const char* name, WFlags fl )
    : QWidget( parent, name, fl )
{
	LED = new QPixmap();
	LEDYellow = new QPixmap();
	LEDGreen = new QPixmap();

	LeftEdge.load("/usr/local/rtmix/images/leftedge.xpm", NULL, DiffuseAlphaDither);
	RightEdge.load("/usr/local/rtmix/images/rightedge.xpm", NULL, DiffuseAlphaDither);

    if ( !name )
	setName( "Warning" );

    resize( 154, 220 ); 
    setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)1, (QSizePolicy::SizeType)1, sizePolicy().hasHeightForWidth() ) );
    setCaption( tr( "Warning" ) );
	setMaximumSize( QSize( 290, 400 ) );

    Warning_WindowLayout = new QGridLayout( this ); 
    Warning_WindowLayout->setSpacing( 6 );
    Warning_WindowLayout->setMargin( 6 );

    WindowLabel = new QLabel( this, "WindowLabel" );
    WindowLabel->setMinimumSize( QSize( 120, 30 ) );
    WindowLabel->setMaximumSize( QSize( 32767, 30 ) );
    QFont WindowLabel_font(  WindowLabel->font() );
    WindowLabel_font.setPointSize( 10 );
    WindowLabel->setFont( WindowLabel_font ); 
    WindowLabel->setText( tr( "Player" ) );
    WindowLabel->setAlignment( int( QLabel::AlignCenter ) );

    Warning_WindowLayout->addMultiCellWidget( WindowLabel, 0, 0, 0, 2 );

	//ADD SPACER
	QSpacerItem* LeftSpacer = new QSpacerItem( 5, 5, QSizePolicy::Minimum, QSizePolicy::Minimum );
	Warning_WindowLayout->addItem( LeftSpacer, 1, 0 );

    WarnLED = new QLED( this, "WarnLED", WResizeNoErase | WRepaintNoErase );
    WarnLED->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7, (QSizePolicy::SizeType)7, WarnLED->sizePolicy().hasHeightForWidth() ) );
	//WarnLED->setUpdatesEnabled( FALSE );
    WarnLED->setMinimumSize( QSize( 120, 120 ) );
    WarnLED->setMaximumSize( QSize( 240, 240 ) );
    WarnLED->setLineWidth( 0 );
    //WarnLED->setPixmap( *LED );
    WarnLED->setScaledContents( TRUE );
    WarnLED->setIndent( 0 );
    QToolTip::add(  WarnLED, tr( "Execute Indicator" ) );

    Warning_WindowLayout->addWidget( WarnLED, 1, 1 );

	//ADD SPACER
	QSpacerItem* RightSpacer = new QSpacerItem( 5, 5, QSizePolicy::Minimum, QSizePolicy::Minimum );
	Warning_WindowLayout->addItem( RightSpacer, 1, 2 );

    LayoutChrono = new QHBoxLayout;
    LayoutChrono->setSpacing( 0 );
    LayoutChrono->setMargin( 0 );

    TimerLeftEdge = new QLabel( this, "TimerLeftEdge" );
    TimerLeftEdge->setMinimumSize( QSize( 10, 45 ) );
    TimerLeftEdge->setMaximumSize( QSize( 10, 32767 ) );
    TimerLeftEdge->setPixmap( LeftEdge );
    TimerLeftEdge->setScaledContents( TRUE );
    LayoutChrono->addWidget( TimerLeftEdge );

    NTime = new QLCDNumber( this, "NTime" );
    NTime->setMinimumSize( QSize( 35, 45 ) );
    NTime->setMaximumSize( QSize( 32767, 32767 ) );
    cg.setColor( QColorGroup::Foreground, QColor( 85, 255, 0) );
    cg.setColor( QColorGroup::Button, QColor( 85, 255, 0) );
    cg.setColor( QColorGroup::Light, QColor( 170, 255, 127) );
    cg.setColor( QColorGroup::Midlight, QColor( 127, 255, 63) );
    cg.setColor( QColorGroup::Dark, QColor( 42, 127, 0) );
    cg.setColor( QColorGroup::Mid, QColor( 57, 170, 0) );
    cg.setColor( QColorGroup::Text, white );
    cg.setColor( QColorGroup::BrightText, white );
    cg.setColor( QColorGroup::ButtonText, white );
    cg.setColor( QColorGroup::Base, black );
    cg.setColor( QColorGroup::Background, black );
    cg.setColor( QColorGroup::Shadow, black );
    cg.setColor( QColorGroup::Highlight, QColor( 0, 0, 128) );
    cg.setColor( QColorGroup::HighlightedText, white );
    pal.setActive( cg );
    pal.setInactive( cg );
    pal.setDisabled( cg );
    NTime->setPalette( pal );
    NTime->setCursor( QCursor( 0 ) );
    NTime->setMouseTracking( FALSE );
    NTime->setFocusPolicy( QLCDNumber::NoFocus );
    NTime->setAcceptDrops( FALSE );
    NTime->setAutoMask( FALSE );
    NTime->setFrameShape( QLCDNumber::NoFrame );
    NTime->setFrameShadow( QLCDNumber::Plain );
    NTime->setLineWidth( 2 );
    NTime->setMargin( 0 );
    NTime->setMidLineWidth( 0 );
    NTime->setSmallDecimalPoint( FALSE );
    NTime->setNumDigits( 5 );
    NTime->setMode( QLCDNumber::DEC );
    NTime->setSegmentStyle( QLCDNumber::Filled );
    //NTime->setProperty( "intValue", 0 );
    NTime->display( "00.00" );
	LayoutChrono->addWidget( NTime );

    TimerRightEdge = new QLabel( this, "TimerRightEdge" );
    TimerRightEdge->setEnabled( TRUE );
    TimerRightEdge->setMinimumSize( QSize( 10, 45 ) );
    TimerRightEdge->setMaximumSize( QSize( 10, 32767 ) );
    TimerRightEdge->setPixmap( RightEdge );
    TimerRightEdge->setScaledContents( TRUE );
    LayoutChrono->addWidget( TimerRightEdge );

    Warning_WindowLayout->addMultiCellLayout( LayoutChrono, 2, 2, 0, 2 );
    Status = false;
    LightStatus = 0;
    EventPointer = new eventCue();
    warnLength = -1;
}

/*  
 *  Destroys the object and frees any allocated resources
 */
Warning_Window::~Warning_Window()
{
    // no need to delete child widgets, Qt does it all for us
}

/*  
 *  Main event handler. Reimplemented to handle application
 *  font changes
 */
bool Warning_Window::event( QEvent* ev )
{
    bool ret = QWidget::event( ev ); 
    if ( ev->type() == QEvent::ApplicationFontChange ) {
	QFont WindowLabel_font(  WindowLabel->font() );
	WindowLabel_font.setPointSize( 10 );
	WindowLabel->setFont( WindowLabel_font ); 
    }
	if ( ev->type() == QEvent::Hide )
	{
     	emit Closed( 0 );
	}
    //if ( ev->type() == QEvent::Resize )
    //{
    //    qDebug( "%d %d", width(), height() );
    //}
    return ret;
}

void Warning_Window::InternalUpdate()
{
    if ( warnLength >= 0 )
    {
        UpdateDisplay( EventPointer, warnLength-- );
        //qDebug("oops");
    }
}

int Warning_Window::UpdateDisplay( struct eventCue *WarningPointer, long timer )
{
	//QString tempvar; //DEBUG

	//timer = timer - 1;
	//WindowLabel->setText( tempvar.setNum( timer, 10 ) ); //DEBUG

	if ( !Status && timer != 0 || EventPointer != WarningPointer )
	{
        Status = true;
        //qDebug("reinit");
        *EventPointer = *WarningPointer;
        warnLength = timer;
		 //WindowLabel->setText( "Here we go!");
	}
	if ( Status && timer == 0 )
	{
		ResetDisplay();
     	Status = 0;
		return 0;
	}

    //if ( Status && timer != 0 && EventPointer != NULL && EventPointer != WarningPointer )
    //{
    //    *EventPointer = *WarningPointer;        
    //}

	//CHECK LIGHT SETTINGS
	if ( EventPointer->countdown != 0 ) //This means it has Countdown behavior
	{
		if ( timer == EventPointer->countdown + EventPointer->lightLength ) //This is first time through
		{
         	WarnLED->setPixmap( *LEDYellow );
			LightStatus = true;
		}
		if ( EventPointer->blinkSpeed != 0 && timer <= EventPointer->lightLength + EventPointer->prepLength && timer > EventPointer->lightLength && float(float(EventPointer->prepLength + EventPointer->lightLength - timer)/ float(EventPointer->blinkSpeed)) == int( (EventPointer->prepLength + EventPointer->lightLength - timer)/EventPointer->blinkSpeed ) ) //blinking zone
		{
        	if ( LightStatus )
			{
				if ( EventPointer->prepLength == (timer - EventPointer->lightLength) && EventPointer->prepLength != EventPointer->countdown || (timer - EventPointer->lightLength) < EventPointer->prepLength )
				{
             		WarnLED->setPixmap( *LED );
					LightStatus = false;
				}
			}
			else if ( !LightStatus )
			{
				WarnLED->setPixmap( *LEDYellow );
				LightStatus = true;
			}
		}
		if ( timer == EventPointer->lightLength && EventPointer->lightLength != 0 )
		{
             	WarnLED->setPixmap( *LEDGreen );
				LightStatus = true;         	
		}
		if ( timer >= EventPointer->lightLength )
		{
			SetDisplay( EventPointer, timer);
		}
	}

	else if ( EventPointer->blinkSpeed != 0 ) //This means it has blinking element in it
	{
		if ( timer == EventPointer->lightLength ) //This is first time through
		{
         	WarnLED->setPixmap( *LEDYellow );
			LightStatus = true;
		}
		if ( timer < EventPointer->lightLength && float(float(EventPointer->lightLength - timer)/float(EventPointer->blinkSpeed)) == int( (EventPointer->lightLength - timer)/EventPointer->blinkSpeed ) ) //blinking zone
		{
        	if ( LightStatus )
			{
             	WarnLED->setPixmap( *LED );
				LightStatus = false;
			}
			else
			{
				WarnLED->setPixmap( *LEDYellow );
				LightStatus = true;
			}
		}
	}
	
	else //Otherwise this is a simple light-up thing
	{
		if ( timer == EventPointer->lightLength ) //This is first time through
		{
         	WarnLED->setPixmap( *LEDGreen );
			LightStatus = true;
		}
	}
	return 0;
}

void Warning_Window::ResetDisplay()
{
	if ( LightStatus )
	{
        WarnLED->setPixmap( *LED );
        LightStatus = 0;
        Status = 0;
        warnLength = -1;
	}
	NTime->display( "00.00" );
}

void Warning_Window::SetName( QString WinName, QString TopLabel )
{
	if ( WinName != "" )
	{
		setCaption( WinName );
	}
	WindowLabel->setText( TopLabel );	
}

void Warning_Window::SetDisplay(struct eventCue *WarningPointer, long timer)
{
    QString tempvar, time;
	long temp, tempval;

	if ( EventPointer->countdown != 0 && EventPointer->countdown != timer )
	{
		temp = EventPointer->lightLength;
	}
	else
	{
     	temp = 0;
	}

	tempvar.setNum( int((timer - temp)/100), 10 );
	if ( timer - temp < 1000 )
	{
	 	tempvar = "0" + tempvar;
	}
	time = tempvar;
	tempval = ( (timer - temp) - int( (( timer - temp) / 100 ) * 100) );
	tempvar.setNum( tempval , 10 );
	if (tempvar.length() == 1)
	{
		tempvar = "0" + tempvar;
	}
	time = time + "." + tempvar;
	NTime->display( time );
}

void Warning_Window::SetImage( QString NamePath, int val )
{

	if ( val == 0 )
	{		
    	LED->load( NamePath, NULL, DiffuseAlphaDither );
		WarnLED->setPixmap( *LED );
	}
   if ( val == 1 )
	{
    	LEDYellow->load( NamePath, NULL, DiffuseAlphaDither );
		//qDebug ( NamePath + " %i ", test );
	}
   if ( val == 2 )
	{
    	LEDGreen->load( NamePath, NULL, DiffuseAlphaDither );
		//qDebug ( NamePath + " %i ", test );
	}
}

void Warning_Window::keyPressEvent ( QKeyEvent *k )
{
    emit KeyPress( k );
}

void Warning_Window::keyReleaseEvent ( QKeyEvent *k )
{
    emit KeyRelease( k );
}

void Warning_Window::SetResizable( bool b )
{
    if ( !b )
    {
        setMinimumSize( size() );
        setMaximumSize( size() );
        //resize( size() );
    }
    else
    {
        setMinimumSize( 154, 220 );
        setMaximumSize( 290, 400 );
    }
}
